@@ -1,8 +1,12 @@
 #include <qpdf/Pipeline.hh>
 
+#include <qpdf/Util.hh>
+
 #include <cstring>
 #include <stdexcept>
 
+using namespace qpdf;
+
 Pipeline::Pipeline(char const* identifier, Pipeline* next) :
     identifier(identifier),
     next_(next)
@@ -12,103 +16,101 @@ Pipeline::Pipeline(char const* identifier, Pipeline* next) :
 Pipeline*
 Pipeline::getNext(bool allow_null)
 {
-    if (!next_ && !allow_null) {
-        throw std::logic_error(
-            this->identifier + ": Pipeline::getNext() called on pipeline with no next");
-    }
+    util::assertion(
+        next_ || allow_null, identifier + ": Pipeline::getNext() called on pipeline with no next");
     return next_;
 }
 
 std::string
 Pipeline::getIdentifier() const
 {
-    return this->identifier;
+    return identifier;
 }
 
 void
 Pipeline::writeCStr(char const* cstr)
 {
-    this->write(cstr, strlen(cstr));
+    write(cstr, strlen(cstr));
 }
 
 void
 Pipeline::writeString(std::string const& str)
 {
-    this->write(str.c_str(), str.length());
+    write(str.c_str(), str.length());
 }
 
 Pipeline&
 Pipeline::operator<<(char const* cstr)
 {
-    this->writeCStr(cstr);
+    writeCStr(cstr);
     return *this;
 }
 
 Pipeline&
 Pipeline::operator<<(std::string const& str)
 {
-    this->writeString(str);
+    writeString(str);
     return *this;
 }
 
 Pipeline&
 Pipeline::operator<<(short i)
 {
-    this->writeString(std::to_string(i));
+    writeString(std::to_string(i));
     return *this;
 }
 
 Pipeline&
 Pipeline::operator<<(int i)
 {
-    this->writeString(std::to_string(i));
+    writeString(std::to_string(i));
     return *this;
 }
 
 Pipeline&
 Pipeline::operator<<(long i)
 {
-    this->writeString(std::to_string(i));
+    writeString(std::to_string(i));
     return *this;
 }
 
 Pipeline&
 Pipeline::operator<<(long long i)
 {
-    this->writeString(std::to_string(i));
+    writeString(std::to_string(i));
     return *this;
 }
 
 Pipeline&
 Pipeline::operator<<(unsigned short i)
 {
-    this->writeString(std::to_string(i));
+    writeString(std::to_string(i));
     return *this;
 }
 
 Pipeline&
 Pipeline::operator<<(unsigned int i)
 {
-    this->writeString(std::to_string(i));
+    writeString(std::to_string(i));
     return *this;
 }
 
 Pipeline&
 Pipeline::operator<<(unsigned long i)
 {
-    this->writeString(std::to_string(i));
+    writeString(std::to_string(i));
     return *this;
 }
 
 Pipeline&
 Pipeline::operator<<(unsigned long long i)
 {
-    this->writeString(std::to_string(i));
+    writeString(std::to_string(i));
     return *this;
 }
 
 void
 Pipeline::write(char const* data, size_t len)
 {
-    this->write(reinterpret_cast<unsigned char const*>(data), len);
+    write(reinterpret_cast<unsigned char const*>(data), len);
 }
