@@ -1,24 +1,31 @@
 #include <qpdf/Pl_Count.hh>
 
 #include <qpdf/QIntC.hh>
+#include <qpdf/Util.hh>
 
-Pl_Count::Members::Members()
+using namespace qpdf;
+
+class Pl_Count::Members
 {
-}
+  public:
+    Members() = default;
+    Members(Members const&) = delete;
+    ~Members() = default;
+
+    // Must be qpdf_offset_t, not size_t, to handle writing more than size_t can handle.
+    qpdf_offset_t count{0};
+    unsigned char last_char{'\0'};
+};
 
 Pl_Count::Pl_Count(char const* identifier, Pipeline* next) :
     Pipeline(identifier, next),
-    m(new Members())
+    m(std::make_unique<Members>())
 {
-    if (!next) {
-        throw std::logic_error("Attempt to create Pl_Count with nullptr as next");
-    }
+    util::assertion(next, "Attempt to create Pl_Count with nullptr as next");
 }
 
-Pl_Count::~Pl_Count() // NOLINT (modernize-use-equals-default)
-{
-    // Must be explicit and not inline -- see QPDF_DLL_CLASS in README-maintainer
-}
+Pl_Count::~Pl_Count() = default;
+// Must be explicit and not inline -- see QPDF_DLL_CLASS in README-maintainer
 
 void
 Pl_Count::write(unsigned char const* buf, size_t len)
