@@ -1,19 +1,30 @@
 #include <qpdf/Pl_QPDFTokenizer.hh>
 
-#include <qpdf/BufferInputSource.hh>
+#include <qpdf/InputSource_private.hh>
+#include <qpdf/Pipeline_private.hh>
 #include <qpdf/QTC.hh>
+
 #include <stdexcept>
 
-Pl_QPDFTokenizer::Members::Members() :
-    filter(nullptr),
-    buf("tokenizer buffer")
+using namespace qpdf;
+
+class Pl_QPDFTokenizer::Members
 {
-}
+  public:
+    Members() = default;
+    Members(Members const&) = delete;
+    ~Members() = default;
+
+    QPDFObjectHandle::TokenFilter* filter{nullptr};
+    QPDFTokenizer tokenizer;
+    std::string buffer;
+    pl::String buf{"pl_tokenizer", nullptr, buffer};
+};
 
 Pl_QPDFTokenizer::Pl_QPDFTokenizer(
     char const* identifier, QPDFObjectHandle::TokenFilter* filter, Pipeline* next) :
     Pipeline(identifier, next),
-    m(new Members)
+    m(std::make_unique<Members>())
 {
     m->filter = filter;
     QPDFObjectHandle::TokenFilter::PipelineAccessor::setPipeline(m->filter, next);
@@ -21,10 +32,8 @@ Pl_QPDFTokenizer::Pl_QPDFTokenizer(
     m->tokenizer.includeIgnorable();
 }
 
-Pl_QPDFTokenizer::~Pl_QPDFTokenizer() // NOLINT (modernize-use-equals-default)
-{
-    // Must be explicit and not inline -- see QPDF_DLL_CLASS in README-maintainer
-}
+// Must be explicit and not inline -- see QPDF_DLL_CLASS in README-maintainer
+Pl_QPDFTokenizer::~Pl_QPDFTokenizer() = default;
 
 void
 Pl_QPDFTokenizer::write(unsigned char const* data, size_t len)
@@ -35,21 +44,17 @@ Pl_QPDFTokenizer::write(unsigned char const* data, size_t len)
 void
 Pl_QPDFTokenizer::finish()
 {
-    m->buf.finish();
-    auto input = std::shared_ptr<InputSource>(
-        // line-break
-        new BufferInputSource("tokenizer data", m->buf.getBuffer(), true));
-
+    auto input = is::OffsetBuffer("tokenizer data", m->buffer);
+    std::string empty;
     while (true) {
-        QPDFTokenizer::Token token =
-            m->tokenizer.readToken(input, "offset " + std::to_string(input->tell()), true);
+        auto token = m->tokenizer.readToken(input, empty, true);
         m->filter->handleToken(token);
         if (token.getType() == QPDFTokenizer::tt_eof) {
             break;
         } else if (token.isWord("ID")) {
             // Read the space after the ID.
             char ch = ' ';
-            input->read(&ch, 1);
+            input.read(&ch, 1);
             m->filter->handleToken(
                 // line-break
                 QPDFTokenizer::Token(QPDFTokenizer::tt_space, std::string(1, ch)));
@@ -59,8 +64,7 @@ Pl_QPDFTokenizer::finish()
     }
     m->filter->handleEOF();
     QPDFObjectHandle::TokenFilter::PipelineAccessor::setPipeline(m->filter, nullptr);
-    Pipeline* next = this->getNext(true);
-    if (next) {
-        next->finish();
+    if (next()) {
+        next()->finish();
     }
 }
