@@ -3,24 +3,34 @@
 #include <qpdf/Pl_StdioFile.hh>
 
 #include <qpdf/QUtil.hh>
+#include <qpdf/Util.hh>
+
 #include <cerrno>
 #include <stdexcept>
 
-Pl_StdioFile::Members::Members(FILE* f) :
-    file(f)
+using namespace qpdf;
+
+class Pl_StdioFile::Members
 {
-}
+  public:
+    Members(FILE* f) :
+        file(f)
+    {
+    }
+    Members(Members const&) = delete;
+    ~Members() = default;
+
+    FILE* file;
+};
 
 Pl_StdioFile::Pl_StdioFile(char const* identifier, FILE* f) :
     Pipeline(identifier, nullptr),
-    m(new Members(f))
+    m(std::make_unique<Members>(f))
 {
 }
 
-Pl_StdioFile::~Pl_StdioFile() // NOLINT (modernize-use-equals-default)
-{
-    // Must be explicit and not inline -- see QPDF_DLL_CLASS in README-maintainer
-}
+// Must be explicit and not inline -- see QPDF_DLL_CLASS in README-maintainer
+Pl_StdioFile::~Pl_StdioFile() = default;
 
 void
 Pl_StdioFile::write(unsigned char const* buf, size_t len)
@@ -40,7 +50,7 @@ Pl_StdioFile::write(unsigned char const* buf, size_t len)
 void
 Pl_StdioFile::finish()
 {
-    if ((fflush(m->file) == -1) && (errno == EBADF)) {
-        throw std::logic_error(this->identifier + ": Pl_StdioFile::finish: stream already closed");
-    }
+    util::assertion(
+        !(fflush(m->file) == -1 && errno == EBADF),
+        identifier + ": Pl_StdioFile::finish: stream already closed");
 }
