@@ -1,18 +1,25 @@
-#include <qpdf/QPDFTokenizer.hh>
+#include <qpdf/QPDFTokenizer_private.hh>
 
 // DO NOT USE ctype -- it is locale dependent for some things, and it's not worth the risk of
 // including it in case it may accidentally be used.
 
+#include <qpdf/InputSource_private.hh>
 #include <qpdf/QIntC.hh>
 #include <qpdf/QPDFExc.hh>
 #include <qpdf/QPDFObjectHandle.hh>
 #include <qpdf/QTC.hh>
 #include <qpdf/QUtil.hh>
+#include <qpdf/Util.hh>
 
 #include <cstdlib>
 #include <cstring>
 #include <stdexcept>
 
+using namespace qpdf;
+
+using Token = QPDFTokenizer::Token;
+using tt = QPDFTokenizer::token_type_e;
+
 static inline bool
 is_delimiter(char ch)
 {
@@ -27,7 +34,7 @@ namespace
     class QPDFWordTokenFinder: public InputSource::Finder
     {
       public:
-        QPDFWordTokenFinder(std::shared_ptr<InputSource> is, std::string const& str) :
+        QPDFWordTokenFinder(InputSource& is, std::string const& str) :
             is(is),
             str(str)
         {
@@ -36,7 +43,7 @@ namespace
         bool check() override;
 
       private:
-        std::shared_ptr<InputSource> is;
+        InputSource& is;
         std::string str;
     };
 } // namespace
@@ -46,23 +53,23 @@ QPDFWordTokenFinder::check()
 {
     // Find a word token matching the given string, preceded by a delimiter, and followed by a
     // delimiter or EOF.
-    QPDFTokenizer tokenizer;
-    QPDFTokenizer::Token t = tokenizer.readToken(is, "finder", true);
-    qpdf_offset_t pos = is->tell();
-    if (!(t == QPDFTokenizer::Token(QPDFTokenizer::tt_word, str))) {
+    Tokenizer tokenizer;
+    tokenizer.nextToken(is, "finder", str.size() + 2);
+    qpdf_offset_t pos = is.tell();
+    if (tokenizer.getType() != tt::tt_word || tokenizer.getValue() != str) {
         QTC::TC("qpdf", "QPDFTokenizer finder found wrong word");
         return false;
     }
-    qpdf_offset_t token_start = is->getLastOffset();
+    qpdf_offset_t token_start = is.getLastOffset();
     char next;
     bool next_okay = false;
-    if (is->read(&next, 1) == 0) {
+    if (is.read(&next, 1) == 0) {
         QTC::TC("qpdf", "QPDFTokenizer inline image at EOF");
         next_okay = true;
     } else {
         next_okay = is_delimiter(next);
     }
-    is->seek(pos, SEEK_SET);
+    is.seek(pos, SEEK_SET);
     if (!next_okay) {
         return false;
     }
@@ -74,10 +81,10 @@ QPDFWordTokenFinder::check()
 }
 
 void
-QPDFTokenizer::reset()
+Tokenizer::reset()
 {
     state = st_before_token;
-    type = tt_bad;
+    type = tt::tt_bad;
     val.clear();
     raw_val.clear();
     error_message = "";
@@ -102,8 +109,13 @@ QPDFTokenizer::Token::Token(token_type_e type, std::string const& value) :
 }
 
 QPDFTokenizer::QPDFTokenizer() :
-    allow_eof(false),
-    include_ignorable(false)
+    m(std::make_unique<qpdf::Tokenizer>())
+{
+}
+
+QPDFTokenizer::~QPDFTokenizer() = default;
+
+Tokenizer::Tokenizer()
 {
     reset();
 }
@@ -111,45 +123,62 @@ QPDFTokenizer::QPDFTokenizer() :
 void
 QPDFTokenizer::allowEOF()
 {
-    this->allow_eof = true;
+    m->allowEOF();
+}
+
+void
+Tokenizer::allowEOF()
+{
+    allow_eof = true;
 }
 
 void
 QPDFTokenizer::includeIgnorable()
 {
-    this->include_ignorable = true;
+    m->includeIgnorable();
 }
 
-bool
-QPDFTokenizer::isSpace(char ch)
+void
+Tokenizer::includeIgnorable()
 {
-    return ((ch == '\0') || QUtil::is_space(ch));
+    include_ignorable = true;
 }
 
 bool
-QPDFTokenizer::isDelimiter(char ch)
+Tokenizer::isSpace(char ch)
+{
+    return (ch == '\0' || util::is_space(ch));
+}
+
+bool
+Tokenizer::isDelimiter(char ch)
 {
     return is_delimiter(ch);
 }
 
 void
 QPDFTokenizer::presentCharacter(char ch)
+{
+    m->presentCharacter(ch);
+}
+
+void
+Tokenizer::presentCharacter(char ch)
 {
     handleCharacter(ch);
 
-    if (this->in_token) {
-        this->raw_val += ch;
+    if (in_token) {
+        raw_val += ch;
     }
 }
 
 void
-QPDFTokenizer::handleCharacter(char ch)
+Tokenizer::handleCharacter(char ch)
 {
-    // State machine is implemented such that the final character may not be handled.  This happens
-    // whenever you have to use a character from the next token to detect the end of the current
-    // token.
+    // In some cases, functions called below may call a second handler. This happens whenever you
+    // have to use a character from the next token to detect the end of the current token.
 
-    switch (this->state) {
+    switch (state) {
     case st_top:
         inTop(ch);
         return;
@@ -244,80 +273,80 @@ QPDFTokenizer::handleCharacter(char ch)
 }
 
 void
-QPDFTokenizer::inTokenReady(char ch)
+Tokenizer::inTokenReady(char ch)
 {
     throw std::logic_error(
         "INTERNAL ERROR: QPDF tokenizer presented character while token is waiting");
 }
 
 void
-QPDFTokenizer::inBeforeToken(char ch)
+Tokenizer::inBeforeToken(char ch)
 {
     // Note: we specifically do not use ctype here.  It is locale-dependent.
     if (isSpace(ch)) {
-        this->before_token = !this->include_ignorable;
-        this->in_token = this->include_ignorable;
-        if (this->include_ignorable) {
-            this->state = st_in_space;
+        before_token = !include_ignorable;
+        in_token = include_ignorable;
+        if (include_ignorable) {
+            state = st_in_space;
         }
     } else if (ch == '%') {
-        this->before_token = !this->include_ignorable;
-        this->in_token = this->include_ignorable;
-        this->state = st_in_comment;
+        before_token = !include_ignorable;
+        in_token = include_ignorable;
+        state = st_in_comment;
     } else {
-        this->before_token = false;
-        this->in_token = true;
+        before_token = false;
+        in_token = true;
         inTop(ch);
     }
 }
 
 void
-QPDFTokenizer::inTop(char ch)
+Tokenizer::inTop(char ch)
 {
     switch (ch) {
     case '(':
-        this->string_depth = 1;
-        this->state = st_in_string;
+        string_depth = 1;
+        state = st_in_string;
         return;
 
     case '<':
-        this->state = st_lt;
+        state = st_lt;
         return;
 
     case '>':
-        this->state = st_gt;
+        state = st_gt;
         return;
 
     case (')'):
-        this->type = tt_bad;
+        type = tt::tt_bad;
         QTC::TC("qpdf", "QPDFTokenizer bad )");
-        this->error_message = "unexpected )";
-        this->state = st_token_ready;
+        error_message = "unexpected )";
+        state = st_token_ready;
         return;
 
     case '[':
-        this->type = tt_array_open;
-        this->state = st_token_ready;
+        type = tt::tt_array_open;
+        state = st_token_ready;
         return;
 
     case ']':
-        this->type = tt_array_close;
-        this->state = st_token_ready;
+        type = tt::tt_array_close;
+        state = st_token_ready;
         return;
 
     case '{':
-        this->type = tt_brace_open;
-        this->state = st_token_ready;
+        type = tt::tt_brace_open;
+        state = st_token_ready;
         return;
 
     case '}':
-        this->type = tt_brace_close;
-        this->state = st_token_ready;
+        type = tt::tt_brace_close;
+        state = st_token_ready;
         return;
 
     case '/':
-        this->state = st_name;
-        this->val += ch;
+        state = st_name;
+        val += ch;
         return;
 
     case '0':
@@ -330,92 +359,92 @@ QPDFTokenizer::inTop(char ch)
     case '7':
     case '8':
     case '9':
-        this->state = st_number;
+        state = st_number;
         return;
 
     case '+':
     case '-':
-        this->state = st_sign;
+        state = st_sign;
         return;
 
     case '.':
-        this->state = st_decimal;
+        state = st_decimal;
         return;
 
     default:
-        this->state = st_literal;
+        state = st_literal;
         return;
     }
 }
 
 void
-QPDFTokenizer::inSpace(char ch)
+Tokenizer::inSpace(char ch)
 {
     // We only enter this state if include_ignorable is true.
     if (!isSpace(ch)) {
-        this->type = tt_space;
-        this->in_token = false;
-        this->char_to_unread = ch;
-        this->state = st_token_ready;
+        type = tt::tt_space;
+        in_token = false;
+        char_to_unread = ch;
+        state = st_token_ready;
     }
 }
 
 void
-QPDFTokenizer::inComment(char ch)
+Tokenizer::inComment(char ch)
 {
     if ((ch == '\r') || (ch == '\n')) {
-        if (this->include_ignorable) {
-            this->type = tt_comment;
-            this->in_token = false;
-            this->char_to_unread = ch;
-            this->state = st_token_ready;
+        if (include_ignorable) {
+            type = tt::tt_comment;
+            in_token = false;
+            char_to_unread = ch;
+            state = st_token_ready;
         } else {
-            this->state = st_before_token;
+            state = st_before_token;
         }
     }
 }
 
 void
-QPDFTokenizer::inString(char ch)
+Tokenizer::inString(char ch)
 {
     switch (ch) {
     case '\\':
-        this->state = st_string_escape;
+        state = st_string_escape;
         return;
 
     case '(':
-        this->val += ch;
-        ++this->string_depth;
+        val += ch;
+        ++string_depth;
         return;
 
     case ')':
-        if (--this->string_depth == 0) {
-            this->type = tt_string;
-            this->state = st_token_ready;
+        if (--string_depth == 0) {
+            type = tt::tt_string;
+            state = st_token_ready;
             return;
         }
 
-        this->val += ch;
+        val += ch;
         return;
 
     case '\r':
         // CR by itself is converted to LF
-        this->val += '\n';
-        this->state = st_string_after_cr;
+        val += '\n';
+        state = st_string_after_cr;
         return;
 
     case '\n':
-        this->val += ch;
+        val += ch;
         return;
 
     default:
-        this->val += ch;
+        val += ch;
         return;
     }
 }
 
 void
-QPDFTokenizer::inName(char ch)
+Tokenizer::inName(char ch)
 {
     if (isDelimiter(ch)) {
         // A C-locale whitespace character or delimiter terminates token.  It is important to unread
@@ -424,120 +453,120 @@ QPDFTokenizer::inName(char ch)
         // though not on any files in the test suite as of this
         // writing.
 
-        this->type = this->bad ? tt_bad : tt_name;
-        this->in_token = false;
-        this->char_to_unread = ch;
-        this->state = st_token_ready;
+        type = bad ? tt::tt_bad : tt::tt_name;
+        in_token = false;
+        char_to_unread = ch;
+        state = st_token_ready;
     } else if (ch == '#') {
-        this->char_code = 0;
-        this->state = st_name_hex1;
+        char_code = 0;
+        state = st_name_hex1;
     } else {
-        this->val += ch;
+        val += ch;
     }
 }
 
 void
-QPDFTokenizer::inNameHex1(char ch)
+Tokenizer::inNameHex1(char ch)
 {
-    this->hex_char = ch;
+    hex_char = ch;
 
-    if (char hval = QUtil::hex_decode_char(ch); hval < '\20') {
-        this->char_code = int(hval) << 4;
-        this->state = st_name_hex2;
+    if (char hval = util::hex_decode_char(ch); hval < '\20') {
+        char_code = int(hval) << 4;
+        state = st_name_hex2;
     } else {
         QTC::TC("qpdf", "QPDFTokenizer bad name 1");
-        this->error_message = "name with stray # will not work with PDF >= 1.2";
+        error_message = "name with stray # will not work with PDF >= 1.2";
         // Use null to encode a bad # -- this is reversed in QPDF_Name::normalizeName.
-        this->val += '\0';
-        this->state = st_name;
+        val += '\0';
+        state = st_name;
         inName(ch);
     }
 }
 
 void
-QPDFTokenizer::inNameHex2(char ch)
+Tokenizer::inNameHex2(char ch)
 {
-    if (char hval = QUtil::hex_decode_char(ch); hval < '\20') {
-        this->char_code |= int(hval);
+    if (char hval = util::hex_decode_char(ch); hval < '\20') {
+        char_code |= int(hval);
     } else {
         QTC::TC("qpdf", "QPDFTokenizer bad name 2");
-        this->error_message = "name with stray # will not work with PDF >= 1.2";
+        error_message = "name with stray # will not work with PDF >= 1.2";
         // Use null to encode a bad # -- this is reversed in QPDF_Name::normalizeName.
-        this->val += '\0';
-        this->val += this->hex_char;
-        this->state = st_name;
+        val += '\0';
+        val += hex_char;
+        state = st_name;
         inName(ch);
         return;
     }
-    if (this->char_code == 0) {
+    if (char_code == 0) {
         QTC::TC("qpdf", "QPDFTokenizer null in name");
-        this->error_message = "null character not allowed in name token";
-        this->val += "#00";
-        this->state = st_name;
-        this->bad = true;
+        error_message = "null character not allowed in name token";
+        val += "#00";
+        state = st_name;
+        bad = true;
     } else {
-        this->val += char(this->char_code);
-        this->state = st_name;
+        val += char(char_code);
+        state = st_name;
     }
 }
 
 void
-QPDFTokenizer::inSign(char ch)
+Tokenizer::inSign(char ch)
 {
-    if (QUtil::is_digit(ch)) {
-        this->state = st_number;
+    if (util::is_digit(ch)) {
+        state = st_number;
     } else if (ch == '.') {
-        this->state = st_decimal;
+        state = st_decimal;
     } else {
-        this->state = st_literal;
+        state = st_literal;
         inLiteral(ch);
     }
 }
 
 void
-QPDFTokenizer::inDecimal(char ch)
+Tokenizer::inDecimal(char ch)
 {
-    if (QUtil::is_digit(ch)) {
-        this->state = st_real;
+    if (util::is_digit(ch)) {
+        state = st_real;
     } else {
-        this->state = st_literal;
+        state = st_literal;
         inLiteral(ch);
     }
 }
 
 void
-QPDFTokenizer::inNumber(char ch)
+Tokenizer::inNumber(char ch)
 {
-    if (QUtil::is_digit(ch)) {
+    if (util::is_digit(ch)) {
     } else if (ch == '.') {
-        this->state = st_real;
+        state = st_real;
     } else if (isDelimiter(ch)) {
-        this->type = tt_integer;
-        this->state = st_token_ready;
-        this->in_token = false;
-        this->char_to_unread = ch;
+        type = tt::tt_integer;
+        state = st_token_ready;
+        in_token = false;
+        char_to_unread = ch;
     } else {
-        this->state = st_literal;
+        state = st_literal;
     }
 }
 
 void
-QPDFTokenizer::inReal(char ch)
+Tokenizer::inReal(char ch)
 {
-    if (QUtil::is_digit(ch)) {
+    if (util::is_digit(ch)) {
     } else if (isDelimiter(ch)) {
-        this->type = tt_real;
-        this->state = st_token_ready;
-        this->in_token = false;
-        this->char_to_unread = ch;
+        type = tt::tt_real;
+        state = st_token_ready;
+        in_token = false;
+        char_to_unread = ch;
     } else {
-        this->state = st_literal;
+        state = st_literal;
     }
 }
 void
-QPDFTokenizer::inStringEscape(char ch)
+Tokenizer::inStringEscape(char ch)
 {
-    this->state = st_in_string;
+    state = st_in_string;
     switch (ch) {
     case '0':
     case '1':
@@ -547,86 +576,86 @@ QPDFTokenizer::inStringEscape(char ch)
     case '5':
     case '6':
     case '7':
-        this->state = st_char_code;
-        this->char_code = 0;
-        this->digit_count = 0;
+        state = st_char_code;
+        char_code = 0;
+        digit_count = 0;
         inCharCode(ch);
         return;
 
     case 'n':
-        this->val += '\n';
+        val += '\n';
         return;
 
     case 'r':
-        this->val += '\r';
+        val += '\r';
         return;
 
     case 't':
-        this->val += '\t';
+        val += '\t';
         return;
 
     case 'b':
-        this->val += '\b';
+        val += '\b';
         return;
 
     case 'f':
-        this->val += '\f';
+        val += '\f';
         return;
 
     case '\n':
         return;
 
     case '\r':
-        this->state = st_string_after_cr;
+        state = st_string_after_cr;
         return;
 
     default:
         // PDF spec says backslash is ignored before anything else
-        this->val += ch;
+        val += ch;
         return;
     }
 }
 
 void
-QPDFTokenizer::inStringAfterCR(char ch)
+Tokenizer::inStringAfterCR(char ch)
 {
-    this->state = st_in_string;
+    state = st_in_string;
     if (ch != '\n') {
         inString(ch);
     }
 }
 
 void
-QPDFTokenizer::inLt(char ch)
+Tokenizer::inLt(char ch)
 {
     if (ch == '<') {
-        this->type = tt_dict_open;
-        this->state = st_token_ready;
+        type = tt::tt_dict_open;
+        state = st_token_ready;
         return;
     }
 
-    this->state = st_in_hexstring;
+    state = st_in_hexstring;
     inHexstring(ch);
 }
 
 void
-QPDFTokenizer::inGt(char ch)
+Tokenizer::inGt(char ch)
 {
     if (ch == '>') {
-        this->type = tt_dict_close;
-        this->state = st_token_ready;
+        type = tt::tt_dict_close;
+        state = st_token_ready;
     } else {
-        this->type = tt_bad;
+        type = tt::tt_bad;
         QTC::TC("qpdf", "QPDFTokenizer bad >");
-        this->error_message = "unexpected >";
-        this->in_token = false;
-        this->char_to_unread = ch;
-        this->state = st_token_ready;
+        error_message = "unexpected >";
+        in_token = false;
+        char_to_unread = ch;
+        state = st_token_ready;
     }
 }
 
 void
-QPDFTokenizer::inLiteral(char ch)
+Tokenizer::inLiteral(char ch)
 {
     if (isDelimiter(ch)) {
         // A C-locale whitespace character or delimiter terminates token.  It is important to unread
@@ -634,91 +663,102 @@ QPDFTokenizer::inLiteral(char ch)
         // stream keyword.  Removing it here could make the stream-reading code break on some files,
         // though not on any files in the test suite as of this writing.
 
-        this->in_token = false;
-        this->char_to_unread = ch;
-        this->state = st_token_ready;
-        this->type = (this->raw_val == "true") || (this->raw_val == "false")
-            ? tt_bool
-            : (this->raw_val == "null" ? tt_null : tt_word);
+        in_token = false;
+        char_to_unread = ch;
+        state = st_token_ready;
+        type = (raw_val == "true") || (raw_val == "false")
+            ? tt::tt_bool
+            : (raw_val == "null" ? tt::tt_null : tt::tt_word);
     }
 }
 
 void
-QPDFTokenizer::inHexstring(char ch)
+Tokenizer::inHexstring(char ch)
 {
-    if (char hval = QUtil::hex_decode_char(ch); hval < '\20') {
-        this->char_code = int(hval) << 4;
-        this->state = st_in_hexstring_2nd;
+    if (char hval = util::hex_decode_char(ch); hval < '\20') {
+        char_code = int(hval) << 4;
+        state = st_in_hexstring_2nd;
 
     } else if (ch == '>') {
-        this->type = tt_string;
-        this->state = st_token_ready;
+        type = tt::tt_string;
+        state = st_token_ready;
 
     } else if (isSpace(ch)) {
         // ignore
 
     } else {
-        this->type = tt_bad;
+        type = tt::tt_bad;
         QTC::TC("qpdf", "QPDFTokenizer bad hexstring character");
-        this->error_message = std::string("invalid character (") + ch + ") in hexstring";
-        this->state = st_token_ready;
+        error_message = std::string("invalid character (") + ch + ") in hexstring";
+        state = st_token_ready;
     }
 }
 
 void
-QPDFTokenizer::inHexstring2nd(char ch)
+Tokenizer::inHexstring2nd(char ch)
 {
-    if (char hval = QUtil::hex_decode_char(ch); hval < '\20') {
-        this->val += char(this->char_code) | hval;
-        this->state = st_in_hexstring;
+    if (char hval = util::hex_decode_char(ch); hval < '\20') {
+        val += char(char_code) | hval;
+        state = st_in_hexstring;
 
     } else if (ch == '>') {
         // PDF spec says odd hexstrings have implicit trailing 0.
-        this->val += char(this->char_code);
-        this->type = tt_string;
-        this->state = st_token_ready;
+        val += char(char_code);
+        type = tt::tt_string;
+        state = st_token_ready;
 
     } else if (isSpace(ch)) {
         // ignore
 
     } else {
-        this->type = tt_bad;
+        type = tt::tt_bad;
         QTC::TC("qpdf", "QPDFTokenizer bad hexstring 2nd character");
-        this->error_message = std::string("invalid character (") + ch + ") in hexstring";
-        this->state = st_token_ready;
+        error_message = std::string("invalid character (") + ch + ") in hexstring";
+        state = st_token_ready;
     }
 }
 
 void
-QPDFTokenizer::inCharCode(char ch)
+Tokenizer::inCharCode(char ch)
 {
+    bool handled = false;
     if (('0' <= ch) && (ch <= '7')) {
-        this->char_code = 8 * this->char_code + (int(ch) - int('0'));
-        if (++(this->digit_count) < 3) {
+        char_code = 8 * char_code + (int(ch) - int('0'));
+        if (++(digit_count) < 3) {
             return;
         }
-        // We've accumulated \ddd.  PDF Spec says to ignore high-order overflow.
+        handled = true;
+    }
+    // We've accumulated \ddd or we have \d or \dd followed by other than an octal digit. The PDF
+    // Spec says to ignore high-order overflow.
+    val += char(char_code % 256);
+    state = st_in_string;
+    if (!handled) {
+        inString(ch);
     }
-    this->val += char(this->char_code % 256);
-    this->state = st_in_string;
-    return;
 }
 
 void
-QPDFTokenizer::inInlineImage(char ch)
+Tokenizer::inInlineImage(char ch)
 {
-    if ((this->raw_val.length() + 1) == this->inline_image_bytes) {
+    if ((raw_val.length() + 1) == inline_image_bytes) {
         QTC::TC("qpdf", "QPDFTokenizer found EI by byte count");
-        this->type = tt_inline_image;
-        this->inline_image_bytes = 0;
-        this->state = st_token_ready;
+        type = tt::tt_inline_image;
+        inline_image_bytes = 0;
+        state = st_token_ready;
     }
 }
 
 void
 QPDFTokenizer::presentEOF()
 {
-    switch (this->state) {
+    m->presentEOF();
+}
+
+void
+Tokenizer::presentEOF()
+{
+    switch (state) {
     case st_name:
     case st_name_hex1:
     case st_name_hex2:
@@ -730,20 +770,20 @@ QPDFTokenizer::presentEOF()
         QTC::TC("qpdf", "QPDFTokenizer EOF reading appendable token");
         // Push any delimiter to the state machine to finish off the final token.
         presentCharacter('\f');
-        this->in_token = true;
+        in_token = true;
         break;
 
     case st_top:
     case st_before_token:
-        this->type = tt_eof;
+        type = tt::tt_eof;
         break;
 
     case st_in_space:
-        this->type = this->include_ignorable ? tt_space : tt_eof;
+        type = include_ignorable ? tt::tt_space : tt::tt_eof;
         break;
 
     case st_in_comment:
-        this->type = this->include_ignorable ? tt_comment : tt_bad;
+        type = include_ignorable ? tt::tt_comment : tt::tt_bad;
         break;
 
     case st_token_ready:
@@ -751,36 +791,44 @@ QPDFTokenizer::presentEOF()
 
     default:
         QTC::TC("qpdf", "QPDFTokenizer EOF reading token");
-        this->type = tt_bad;
-        this->error_message = "EOF while reading token";
+        type = tt::tt_bad;
+        error_message = "EOF while reading token";
     }
-    this->state = st_token_ready;
+    state = st_token_ready;
 }
 
 void
 QPDFTokenizer::expectInlineImage(std::shared_ptr<InputSource> input)
 {
-    if (this->state == st_token_ready) {
-        reset();
-    } else if (this->state != st_before_token) {
-        throw std::logic_error("QPDFTokenizer::expectInlineImage called"
-                               " when tokenizer is in improper state");
-    }
-    findEI(input);
-    this->before_token = false;
-    this->in_token = true;
-    this->state = st_inline_image;
+    m->expectInlineImage(*input);
 }
 
 void
-QPDFTokenizer::findEI(std::shared_ptr<InputSource> input)
+QPDFTokenizer::expectInlineImage(InputSource& input)
 {
-    if (!input.get()) {
-        return;
-    }
+    m->expectInlineImage(input);
+}
 
-    qpdf_offset_t last_offset = input->getLastOffset();
-    qpdf_offset_t pos = input->tell();
+void
+Tokenizer::expectInlineImage(InputSource& input)
+{
+    if (state == st_token_ready) {
+        reset();
+    } else if (state != st_before_token) {
+        throw std::logic_error(
+            "QPDFTokenizer::expectInlineImage called when tokenizer is in improper state");
+    }
+    findEI(input);
+    before_token = false;
+    in_token = true;
+    state = st_inline_image;
+}
+
+void
+Tokenizer::findEI(InputSource& input)
+{
+    qpdf_offset_t last_offset = input.getLastOffset();
+    qpdf_offset_t pos = input.tell();
 
     // Use QPDFWordTokenFinder to find EI surrounded by delimiters. Then read the next several
     // tokens or up to EOF. If we find any suspicious-looking or tokens, this is probably still part
@@ -793,12 +841,12 @@ QPDFTokenizer::findEI(std::shared_ptr<InputSource> input)
     bool first_try = true;
     while (!okay) {
         QPDFWordTokenFinder f(input, "EI");
-        if (!input->findFirst("EI", input->tell(), 0, f)) {
+        if (!input.findFirst("EI", input.tell(), 0, f)) {
             break;
         }
-        this->inline_image_bytes = QIntC::to_size(input->tell() - pos - 2);
+        inline_image_bytes = QIntC::to_size(input.tell() - pos - 2);
 
-        QPDFTokenizer check;
+        Tokenizer check;
         bool found_bad = false;
         // Look at the next 10 tokens or up to EOF. The next inline image's image data would look
         // like bad tokens, but there will always be at least 10 tokens between one inline image's
@@ -806,13 +854,13 @@ QPDFTokenizer::findEI(std::shared_ptr<InputSource> input)
         // all required as well as a BI and ID. If we get 10 good tokens in a row or hit EOF, we can
         // be pretty sure we've found the actual EI.
         for (int i = 0; i < 10; ++i) {
-            QPDFTokenizer::Token t = check.readToken(input, "checker", true);
-            token_type_e type = t.getType();
-            if (type == tt_eof) {
+            check.nextToken(input, "checker");
+            auto typ = check.getType();
+            if (typ == tt::tt_eof) {
                 okay = true;
-            } else if (type == tt_bad) {
+            } else if (typ == tt::tt_bad) {
                 found_bad = true;
-            } else if (t.isWord()) {
+            } else if (typ == tt::tt_word) {
                 // The qpdf tokenizer lumps alphabetic and otherwise uncategorized characters into
                 // "words". We recognize strings of alphabetic characters as potential valid
                 // operators for purposes of telling whether we're in valid content or not. It's not
@@ -821,13 +869,12 @@ QPDFTokenizer::findEI(std::shared_ptr<InputSource> input)
                 bool found_alpha = false;
                 bool found_non_printable = false;
                 bool found_other = false;
-                for (char ch: t.getValue()) {
-                    if (((ch >= 'a') && (ch <= 'z')) || ((ch >= 'A') && (ch <= 'Z')) ||
-                        (ch == '*')) {
+                for (char ch: check.getValue()) {
+                    if ((ch >= 'a' && ch <= 'z') || (ch >= 'A' && ch <= 'Z') || (ch == '*')) {
                         // Treat '*' as alpha since there are valid PDF operators that contain *
                         // along with alphabetic characters.
                         found_alpha = true;
-                    } else if ((static_cast<signed char>(ch) < 32) && (!isSpace(ch))) {
+                    } else if (static_cast<signed char>(ch) < 32 && !isSpace(ch)) {
                         // Compare ch as a signed char so characters outside of 7-bit will be < 0.
                         found_non_printable = true;
                         break;
@@ -854,22 +901,28 @@ QPDFTokenizer::findEI(std::shared_ptr<InputSource> input)
         QTC::TC("qpdf", "QPDFTokenizer found EI after more than one try");
     }
 
-    input->seek(pos, SEEK_SET);
-    input->setLastOffset(last_offset);
+    input.seek(pos, SEEK_SET);
+    input.setLastOffset(last_offset);
 }
 
 bool
 QPDFTokenizer::getToken(Token& token, bool& unread_char, char& ch)
 {
-    bool ready = (this->state == st_token_ready);
-    unread_char = !this->in_token && !this->before_token;
-    ch = this->char_to_unread;
-    if (ready) {
-        token = (!(this->type == tt_name || this->type == tt_string))
-            ? Token(this->type, this->raw_val, this->raw_val, this->error_message)
-            : Token(this->type, this->val, this->raw_val, this->error_message);
+    return m->getToken(token, unread_char, ch);
+}
 
-        this->reset();
+bool
+Tokenizer::getToken(Token& token, bool& unread_char, char& ch)
+{
+    bool ready = (state == st_token_ready);
+    unread_char = !in_token && !before_token;
+    ch = char_to_unread;
+    if (ready) {
+        token = (!(type == tt::tt_name || type == tt::tt_string))
+            ? Token(type, raw_val, raw_val, error_message)
+            : Token(type, val, raw_val, error_message);
+
+        reset();
     }
     return ready;
 }
@@ -877,12 +930,31 @@ QPDFTokenizer::getToken(Token& token, bool& unread_char, char& ch)
 bool
 QPDFTokenizer::betweenTokens()
 {
-    return this->before_token;
+    return m->betweenTokens();
+}
+
+bool
+Tokenizer::betweenTokens()
+{
+    return before_token;
 }
 
 QPDFTokenizer::Token
 QPDFTokenizer::readToken(
     InputSource& input, std::string const& context, bool allow_bad, size_t max_len)
+{
+    return m->readToken(input, context, allow_bad, max_len);
+}
+
+QPDFTokenizer::Token
+QPDFTokenizer::readToken(
+    std::shared_ptr<InputSource> input, std::string const& context, bool allow_bad, size_t max_len)
+{
+    return m->readToken(*input, context, allow_bad, max_len);
+}
+
+QPDFTokenizer::Token
+Tokenizer::readToken(InputSource& input, std::string const& context, bool allow_bad, size_t max_len)
 {
     nextToken(input, context, max_len);
 
@@ -891,14 +963,14 @@ QPDFTokenizer::readToken(
     char char_to_unread;
     getToken(token, unread_char, char_to_unread);
 
-    if (token.getType() == tt_bad) {
+    if (token.getType() == tt::tt_bad) {
         if (allow_bad) {
             QTC::TC("qpdf", "QPDFTokenizer allowing bad token");
         } else {
             throw QPDFExc(
                 qpdf_e_damaged_pdf,
                 input.getName(),
-                context,
+                context.empty() ? "offset " + std::to_string(input.getLastOffset()) : context,
                 input.getLastOffset(),
                 token.getErrorMessage());
         }
@@ -906,56 +978,49 @@ QPDFTokenizer::readToken(
     return token;
 }
 
-QPDFTokenizer::Token
-QPDFTokenizer::readToken(
-    std::shared_ptr<InputSource> input, std::string const& context, bool allow_bad, size_t max_len)
-{
-    return readToken(*input, context, allow_bad, max_len);
-}
-
 bool
-QPDFTokenizer::nextToken(InputSource& input, std::string const& context, size_t max_len)
+Tokenizer::nextToken(InputSource& input, std::string const& context, size_t max_len)
 {
-    if (this->state != st_inline_image) {
+    if (state != st_inline_image) {
         reset();
     }
     qpdf_offset_t offset = input.fastTell();
 
-    while (this->state != st_token_ready) {
+    while (state != st_token_ready) {
         char ch;
         if (!input.fastRead(ch)) {
             presentEOF();
 
-            if ((this->type == tt_eof) && (!this->allow_eof)) {
+            if ((type == tt::tt_eof) && (!allow_eof)) {
                 // Nothing in the qpdf library calls readToken without allowEOF anymore, so this
                 // case is not exercised.
-                this->type = tt_bad;
-                this->error_message = "unexpected EOF";
+                type = tt::tt_bad;
+                error_message = "unexpected EOF";
                 offset = input.getLastOffset();
             }
         } else {
             handleCharacter(ch);
-            if (this->before_token) {
+            if (before_token) {
                 ++offset;
             }
-            if (this->in_token) {
-                this->raw_val += ch;
+            if (in_token) {
+                raw_val += ch;
             }
-            if (max_len && (this->raw_val.length() >= max_len) && (this->state != st_token_ready)) {
+            if (max_len && (raw_val.length() >= max_len) && (state != st_token_ready)) {
                 // terminate this token now
                 QTC::TC("qpdf", "QPDFTokenizer block long token");
-                this->type = tt_bad;
-                this->state = st_token_ready;
-                this->error_message = "exceeded allowable length while reading token";
+                type = tt::tt_bad;
+                state = st_token_ready;
+                error_message = "exceeded allowable length while reading token";
             }
         }
     }
 
-    input.fastUnread(!this->in_token && !this->before_token);
+    input.fastUnread(!in_token && !before_token);
 
-    if (this->type != tt_eof) {
+    if (type != tt::tt_eof) {
         input.setLastOffset(offset);
     }
 
-    return this->error_message.empty();
+    return error_message.empty();
 }
