@@ -23,7 +23,7 @@ main()
         }
         decode.finish();
     } catch (std::exception& e) {
-        std::cerr << e.what() << std::endl;
+        std::cerr << e.what() << '\n';
         exit(2);
     }
 
