@@ -29,16 +29,16 @@ main(int, char*[])
         "1234567890123456789012345678901234567890");
     MD5 a;
     a.encodeFile("md5.in");
-    std::cout << a.unparse() << std::endl;
+    std::cout << a.unparse() << '\n';
     MD5 b;
     b.encodeFile("md5.in", 100);
-    std::cout << b.unparse() << std::endl;
+    std::cout << b.unparse() << '\n';
 
-    std::cout << MD5::checkDataChecksum("900150983cd24fb0d6963f7d28e17f72", "abc", 3) << std::endl
-              << MD5::checkFileChecksum("5f4b4321873433daae578f85c72f9e74", "md5.in") << std::endl
-              << MD5::checkFileChecksum("6f4b4321873433daae578f85c72f9e74", "md5.in") << std::endl
-              << MD5::checkDataChecksum("000150983cd24fb0d6963f7d28e17f72", "abc", 3) << std::endl
-              << MD5::checkFileChecksum("6f4b4321873433daae578f85c72f9e74", "glerbl") << std::endl;
+    std::cout << MD5::checkDataChecksum("900150983cd24fb0d6963f7d28e17f72", "abc", 3) << '\n'
+              << MD5::checkFileChecksum("5f4b4321873433daae578f85c72f9e74", "md5.in") << '\n'
+              << MD5::checkFileChecksum("6f4b4321873433daae578f85c72f9e74", "md5.in") << '\n'
+              << MD5::checkDataChecksum("000150983cd24fb0d6963f7d28e17f72", "abc", 3) << '\n'
+              << MD5::checkFileChecksum("6f4b4321873433daae578f85c72f9e74", "glerbl") << '\n';
 
     Pl_Discard d;
     Pl_MD5 p("MD5", &d);
@@ -63,7 +63,7 @@ main(int, char*[])
                 p2.write(buf, len);
                 if (i == 1) {
                     // Partial digest -- resets after each call to write
-                    std::cout << p.getHexDigest() << std::endl;
+                    std::cout << p.getHexDigest() << '\n';
                 }
             }
         }
@@ -72,10 +72,10 @@ main(int, char*[])
         p2.finish();
         // Make sure calling getHexDigest twice with no intervening
         // writes results in the same result each time.
-        std::cout << p.getHexDigest() << std::endl;
-        std::cout << p.getHexDigest() << std::endl;
+        std::cout << p.getHexDigest() << '\n';
+        std::cout << p.getHexDigest() << '\n';
     }
-    std::cout << p2.getHexDigest() << std::endl;
+    std::cout << p2.getHexDigest() << '\n';
 
     return 0;
 }
