@@ -2,17 +2,16 @@
 #include <qpdf/Pl_StdioFile.hh>
 #include <qpdf/QUtil.hh>
 
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
 #include <iostream>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 
 int
 main(int argc, char* argv[])
 {
     if (argc != 4) {
-        std::cerr << "Usage: runlength {-encode|-decode} infile outfile"
-                  << std::endl;
+        std::cerr << "Usage: runlength {-encode|-decode} infile outfile" << '\n';
         exit(2);
     }
 
@@ -25,10 +24,7 @@ main(int argc, char* argv[])
     Pl_StdioFile out("stdout", outfile);
     unsigned char buf[100];
     bool done = false;
-    Pl_RunLength rl(
-        "runlength",
-        &out,
-        (encode ? Pl_RunLength::a_encode : Pl_RunLength::a_decode));
+    Pl_RunLength rl("runlength", &out, (encode ? Pl_RunLength::a_encode : Pl_RunLength::a_decode));
     while (!done) {
         size_t len = fread(buf, 1, sizeof(buf), infile);
         if (len <= 0) {
