@@ -13,19 +13,17 @@
 void
 usage()
 {
-    std::cerr << "Usage: test_parsedoffset INPUT.pdf" << std::endl;
+    std::cerr << "Usage: test_parsedoffset INPUT.pdf" << '\n';
 }
 
 std::string
 make_objdesc(qpdf_offset_t offset, QPDFObjectHandle obj)
 {
     std::stringstream ss;
-    ss << "offset = " << offset << " (0x" << std::hex << offset << std::dec
-       << "), ";
+    ss << "offset = " << offset << " (0x" << std::hex << offset << std::dec << "), ";
 
     if (obj.isIndirect()) {
-        ss << "indirect " << obj.getObjectID() << "/" << obj.getGeneration()
-           << ", ";
+        ss << "indirect " << obj.getObjectID() << "/" << obj.getGeneration() << ", ";
     } else {
         ss << "direct, ";
     }
@@ -42,8 +40,7 @@ walk(
     std::vector<std::vector<std::pair<qpdf_offset_t, std::string>>>& result)
 {
     qpdf_offset_t offset = obj.getParsedOffset();
-    std::pair<qpdf_offset_t, std::string> p =
-        std::make_pair(offset, make_objdesc(offset, obj));
+    std::pair<qpdf_offset_t, std::string> p = std::make_pair(offset, make_objdesc(offset, obj));
 
     if (result.size() < stream_number + 1) {
         result.resize(stream_number + 1);
@@ -73,18 +70,16 @@ walk(
 }
 
 void
-process(
-    std::string fn,
-    std::vector<std::vector<std::pair<qpdf_offset_t, std::string>>>& result)
+process(std::string fn, std::vector<std::vector<std::pair<qpdf_offset_t, std::string>>>& result)
 {
     QPDF qpdf;
     qpdf.processFile(fn.c_str());
     std::map<QPDFObjGen, QPDFXRefEntry> xrefs = qpdf.getXRefTable();
 
     for (auto const& oh: qpdf.getAllObjects()) {
-        if (xrefs.count(oh.getObjGen()) == 0) {
+        if (!xrefs.contains(oh.getObjGen())) {
             std::cerr << oh.getObjectID() << "/" << oh.getGeneration()
-                      << " is not found in xref table" << std::endl;
+                      << " is not found in xref table" << '\n';
             std::exit(2);
         }
 
@@ -93,8 +88,8 @@ process(
 
         switch (xref.getType()) {
         case 0:
-            std::cerr << oh.getObjectID() << "/" << oh.getGeneration()
-                      << " xref entry is free" << std::endl;
+            std::cerr << oh.getObjectID() << "/" << oh.getGeneration() << " xref entry is free"
+                      << '\n';
             std::exit(2);
         case 1:
             stream_number = 0;
@@ -103,7 +98,7 @@ process(
             stream_number = static_cast<size_t>(xref.getObjStreamNumber());
             break;
         default:
-            std::cerr << "unknown xref entry type" << std::endl;
+            std::cerr << "unknown xref entry type" << '\n';
             std::exit(2);
         }
 
@@ -125,26 +120,25 @@ main(int argc, char* argv[])
         process(argv[1], table);
 
         for (size_t i = 0; i < table.size(); ++i) {
-            if (table[i].size() == 0) {
+            if (table[i].empty()) {
                 continue;
             }
 
             std::sort(table[i].begin(), table[i].end());
             if (i == 0) {
-                std::cout << "--- objects not in streams ---" << std::endl;
+                std::cout << "--- objects not in streams ---" << '\n';
             } else {
-                std::cout << "--- objects in stream " << i << " ---"
-                          << std::endl;
+                std::cout << "--- objects in stream " << i << " ---" << '\n';
             }
 
             for (auto const& iter: table[i]) {
-                std::cout << iter.second << std::endl;
+                std::cout << iter.second << '\n';
             }
         }
 
-        std::cout << "succeeded" << std::endl;
+        std::cout << "succeeded" << '\n';
     } catch (std::exception& e) {
-        std::cerr << e.what() << std::endl;
+        std::cerr << e.what() << '\n';
         std::exit(2);
     }
 
